@@ -14,9 +14,13 @@ import statistics
 from abc import abstractmethod
 from typing import TYPE_CHECKING, Any, Callable, TypeVar
 
+import pynguin.utils.opcodes as op
+from pynguin.slicer.dynamicslicer import AssertionSlicer, DynamicSlicer
 from pynguin.testcase.execution import ExecutionTrace
+from pynguin.testcase.statement import Statement
 
 if TYPE_CHECKING:
+    from pynguin.slicer.dynamicslicer import SlicingCriterion
     from pynguin.testcase.execution import ExecutionResult, KnownData, TestCaseExecutor
 
 
@@ -233,7 +237,8 @@ class LineTestSuiteFitnessFunction(TestSuiteFitnessFunction):
     def compute_fitness(self, individual) -> float:
         results = self._run_test_suite_chromosome(individual)
         merged_trace = analyze_results(results)
-        existing_lines = self._executor.tracer.get_known_data().existing_lines
+        tracer = self._executor.tracer
+        existing_lines = tracer.get_known_data().existing_lines
         return len(existing_lines) - len(merged_trace.covered_line_ids)
 
     def compute_is_covered(self, individual) -> bool:
@@ -250,6 +255,33 @@ class LineTestSuiteFitnessFunction(TestSuiteFitnessFunction):
         return False
 
 
+class StatementCheckedTestSuiteFitnessFunction(TestSuiteFitnessFunction):
+    """A fitness function based on lines included in the backward
+    slice of each statement of a test suite."""
+
+    def compute_fitness(self, individual) -> float:
+        results = self._run_test_suite_chromosome(individual)
+        merged_trace = analyze_results(results)
+        tracer = self._executor.tracer
+
+        return len(tracer.get_known_data().existing_lines) - len(
+            merged_trace.checked_lines
+        )
+
+    def compute_is_covered(self, individual) -> bool:
+        results = self._run_test_suite_chromosome(individual)
+        merged_trace = analyze_results(results)
+        tracer = self._executor.tracer
+
+        return compute_checked_coverage_statement_fitness_is_covered(
+            merged_trace,
+            tracer.get_known_data(),
+        )
+
+    def is_maximisation_function(self) -> bool:
+        return False
+
+
 class CoverageFunction:  # pylint:disable=too-few-public-methods
     """Interface for a coverage function."""
 
@@ -318,6 +350,65 @@ class TestCaseLineCoverageFunction(TestCaseCoverageFunction):
         return compute_line_coverage(merged_trace, tracer.get_known_data())
 
 
+class TestSuiteStatementCheckedCoverageFunction(TestSuiteCoverageFunction):
+    """Computes checked coverage on the statements of test suites."""
+
+    def compute_coverage(self, individual) -> float:
+        results = self._run_test_suite_chromosome(individual)
+        merged_trace = analyze_results(results)
+        tracer = self._executor.tracer
+
+        existing = len(tracer.get_known_data().existing_lines)
+
+        if existing == 0:
+            # Nothing to cover => everything is covered.
+            coverage = 1.0
+        else:
+            covered = len(merged_trace.checked_lines)
+            coverage = covered / existing
+        assert 0.0 <= coverage <= 1.0, "Coverage must be in [0,1]"
+        return coverage
+
+
+class TestCaseStatementCheckedCoverageFunction(TestCaseCoverageFunction):
+    """Computes checked coverage on the statements of test cases."""
+
+    def compute_coverage(self, individual) -> float:
+        result = self._run_test_case_chromosome(individual)
+        merged_trace = analyze_results([result])
+        tracer = self._executor.tracer
+        existing = len(tracer.get_known_data().existing_lines)
+
+        if existing == 0:
+            # Nothing to cover => everything is covered.
+            coverage = 1.0
+        else:
+            covered = len(merged_trace.checked_lines)
+            coverage = covered / existing
+        assert 0.0 <= coverage <= 1.0, "Coverage must be in [0,1]"
+        return coverage
+
+
+class TestSuiteAssertionCheckedCoverageFunction(TestSuiteCoverageFunction):
+    """Computes checked coverage on test suites with assertions."""
+
+    def compute_coverage(self, individual) -> float:
+        results = self._run_test_suite_chromosome(individual)
+        merged_trace = analyze_results(results)
+        tracer = self._executor.tracer
+        return compute_assertion_checked_coverage(merged_trace, tracer.get_known_data())
+
+
+class TestCaseAssertionCheckedCoverageFunction(TestCaseCoverageFunction):
+    """Computes checked coverage on test cases with assertions."""
+
+    def compute_coverage(self, individual) -> float:
+        result = self._run_test_case_chromosome(individual)
+        merged_trace = analyze_results([result])
+        tracer = self._executor.tracer
+        return compute_assertion_checked_coverage(merged_trace, tracer.get_known_data())
+
+
 class ComputationCache:
     """Caches computation results and computes values on demand."""
 
@@ -715,6 +806,21 @@ def compute_line_coverage_fitness_is_covered(
     return len(trace.covered_line_ids) == len(known_data.existing_lines)
 
 
+def compute_checked_coverage_statement_fitness_is_covered(
+    trace: ExecutionTrace, known_data: KnownData
+) -> bool:
+    """Computes if all lines and code objects are checked by a return statement.
+
+    Args:
+        trace: The execution trace
+        known_data: All known data
+
+    Returns:
+        True, if all lines were checked by a return, false otherwise
+    """
+    return len(trace.checked_lines) == len(known_data.existing_lines)
+
+
 def compute_branch_coverage(trace: ExecutionTrace, known_data: KnownData) -> float:
     """Computes branch coverage on bytecode instructions which should equal
     decision coverage on source.
@@ -771,6 +877,129 @@ def compute_line_coverage(trace: ExecutionTrace, known_data: KnownData) -> float
     return coverage
 
 
+def _cleanse_included_implicit_return_none(
+    known_data, statement_checked_lines, statement_slice
+):
+    if (
+        # check if the last included instructions before the store
+        # are a "return None"
+        len(statement_slice) >= 3
+        and statement_slice[-3].opcode == op.LOAD_CONST
+        and statement_slice[-3].arg is None
+        and statement_slice[-2].opcode == op.RETURN_VALUE
+    ):
+        if (
+            # check if the "return None" is implicit or explicit
+            len(statement_slice) != 3
+            and statement_slice[-4].lineno != statement_slice[-3].lineno
+        ):
+            statement_checked_lines.remove(
+                DynamicSlicer.get_line_id_by_instruction(
+                    statement_slice[-3], known_data
+                )
+            )
+
+
+def compute_statement_checked_lines(
+    statements: list[Statement],
+    trace: ExecutionTrace,
+    known_data: KnownData,
+    statement_slicing_criteria: dict[int, SlicingCriterion],
+) -> set[int]:
+    """Computes checked coverage on bytecode instructions.
+    Each statement can be sliced, returning a list of instructions
+    that are checked by the return value of the statement.
+    If we combine all lists of instructions returned by slicing all statements,
+    we get the combined dynamic slice of the test execution's statements.
+    We then can map all instructions inside the slice to lines
+    that are checked covered of the module under test.
+
+    Args:
+        statements: The sliced instructions
+        trace: The execution trace
+        known_data: All known data
+        statement_slicing_criteria: a dictionary of statement positions
+            and its slicing criteria
+
+    Returns:
+        The checked line ids of lines checked by the statements
+    """
+    known_code_objects = known_data.existing_code_objects
+    dynamic_slicer = DynamicSlicer(known_code_objects)
+    checked_lines_ids = set()
+    for statement in statements:
+        if statement.get_position() not in statement_slicing_criteria:
+            # if there is no slicing criterion there was an exception during
+            # the test case execution and the latter statements after the one
+            # with an exception will never be executed,
+            # thus having no slicing criterion
+            break
+        statement_slice = dynamic_slicer.slice(
+            trace,
+            statement_slicing_criteria[statement.get_position()],
+        )
+        statement_checked_lines = DynamicSlicer.map_instructions_to_lines(
+            statement_slice, known_data
+        )
+
+        _cleanse_included_implicit_return_none(
+            known_data, statement_checked_lines, statement_slice
+        )
+
+        checked_lines_ids.update(statement_checked_lines)
+    return checked_lines_ids
+
+
+def compute_assertion_checked_coverage(
+    trace: ExecutionTrace, known_data: KnownData
+) -> float:
+    """Computes checked coverage on bytecode instructions.
+    Each assertion can be sliced, returning a list of instructions
+    that are checked by an assertion.
+    If we combine all lists of instructions returned by slicing all assertions,
+    we get the combined dynamic slice of the test execution's assertions.
+    We then can map all instructions inside the slice to lines
+    that are checked covered of the module under test.
+    To calculate the coverage we can then divide the amount of lines checked
+    covered through the test execution by the lines overall available in the
+    module under test.
+
+    Args:
+        trace: The execution trace
+        known_data: All known data
+
+    Returns:
+        The computed coverage value
+    """
+    existing = len(known_data.existing_lines)
+
+    if existing == 0:
+        # Nothing to cover => everything is covered.
+        coverage = 1.0
+    else:
+        assertion_slicer = AssertionSlicer(known_data.existing_code_objects)
+        checked_instructions = []
+        for executed_assertion in trace.executed_assertions:
+            assertion_checked_instructions = assertion_slicer.slice_assertion(
+                executed_assertion, trace
+            )
+            executed_assertion.assertion.checked_instructions.extend(
+                assertion_checked_instructions
+            )
+            # checked at any point by the assertion of a statement
+            checked_instructions.extend(assertion_checked_instructions)
+
+        # reduce coverage to lines instead of instructions
+        checked_lines = DynamicSlicer.map_instructions_to_lines(
+            checked_instructions, known_data
+        )
+
+        covered = len(checked_lines)
+        coverage = covered / existing
+    assert 0.0 <= coverage <= 1.0, "Coverage must be in [0,1]"
+    return coverage
+
+
 def compare(fitness_1: float, fitness_2: float) -> int:
     """Compare the two specified values.
 
