@@ -9,6 +9,7 @@ from __future__ import annotations
 
 import time
 from abc import abstractmethod
+from statistics import mean
 from typing import TYPE_CHECKING, Generic, Iterable, TypeVar
 
 from ordered_set import OrderedSet
@@ -23,13 +24,13 @@ if TYPE_CHECKING:
     import pynguin.ga.testcasechromosome as tcc
     import pynguin.generation.searchobserver as so
     import pynguin.testcase.testfactory as tf
+    from pynguin.analyses.module import ModuleTestCluster
     from pynguin.ga.operators.crossover.crossover import CrossOverFunction
     from pynguin.ga.operators.ranking.rankingfunction import RankingFunction
     from pynguin.ga.operators.selection.selection import SelectionFunction
     from pynguin.generation.stoppingconditions.stoppingcondition import (
         StoppingCondition,
     )
-    from pynguin.setup.testcluster import TestCluster
     from pynguin.testcase.execution import TestCaseExecutor
 
 A = TypeVar("A", bound=arch.Archive)  # pylint:disable=invalid-name
@@ -44,10 +45,10 @@ class TestGenerationStrategy(
         self._archive: A
         self._chromosome_factory: cf.ChromosomeFactory
         self._executor: TestCaseExecutor
-        self._test_cluster: TestCluster
+        self._test_cluster: ModuleTestCluster
         self._test_factory: tf.TestFactory
         self._selection_function: SelectionFunction
-        self._stopping_condition: StoppingCondition
+        self._stopping_conditions: list[StoppingCondition]
         self._crossover_function: CrossOverFunction
         self._ranking_function: RankingFunction
         self._test_case_fitness_functions: OrderedSet[
@@ -89,7 +90,7 @@ class TestGenerationStrategy(
         self._executor = executor
 
     @property
-    def test_cluster(self) -> TestCluster:
+    def test_cluster(self) -> ModuleTestCluster:
         """Provide the test cluster.
 
         Returns:
@@ -98,7 +99,7 @@ class TestGenerationStrategy(
         return self._test_cluster
 
     @test_cluster.setter
-    def test_cluster(self, test_cluster: TestCluster) -> None:
+    def test_cluster(self, test_cluster: ModuleTestCluster) -> None:
         self._test_cluster = test_cluster
 
     @property
@@ -148,17 +149,17 @@ class TestGenerationStrategy(
         self._selection_function = selection_function
 
     @property
-    def stopping_condition(self) -> StoppingCondition:
-        """Provides the used stopping condition.
+    def stopping_conditions(self) -> list[StoppingCondition]:
+        """Provides the used stopping conditions.
 
         Returns:
             The used stopping condition
         """
-        return self._stopping_condition
+        return self._stopping_conditions
 
-    @stopping_condition.setter
-    def stopping_condition(self, stopping_condition: StoppingCondition) -> None:
-        self._stopping_condition = stopping_condition
+    @stopping_conditions.setter
+    def stopping_conditions(self, stopping_conditions: list[StoppingCondition]) -> None:
+        self._stopping_conditions = stopping_conditions
 
     @property
     def crossover_function(self) -> CrossOverFunction:
@@ -313,15 +314,14 @@ class TestGenerationStrategy(
         """Checks if there are still resources left, e.g., time or test case executions.
 
         Returns:
-            Whether or not there are resources left.
+            Whether there are resources left.
         """
-        return not self._stopping_condition.is_fulfilled()
+        return all(not sc.is_fulfilled() for sc in self._stopping_conditions)
 
     def progress(self) -> float:
         """Provides the progress of the search.
+        Averages the progress of all stopping conditions.
 
         Returns:
             A value in [0,1]."""
-        limit = self._stopping_condition.limit()
-        assert limit > 0.0
-        return self._stopping_condition.current_value() / limit
+        return mean(sc.current_value() / sc.limit() for sc in self._stopping_conditions)
