@@ -11,8 +11,6 @@ import dataclasses
 import os
 from pathlib import Path
 
-import black
-
 import pynguin.ga.chromosomevisitor as cv
 import pynguin.testcase.testcase_to_ast as tc_to_ast
 import pynguin.utils.namingscope as ns
@@ -172,20 +170,26 @@ class PyTestChromosomeToAstVisitor(cv.ChromosomeVisitor):
         return ast.Module(body=import_nodes + functions, type_ignores=[])
 
 
-def save_module_to_file(module: ast.Module, path: str | os.PathLike) -> None:
+def save_module_to_file(
+    module: ast.Module, path: str | os.PathLike, format_with_black: bool = True
+) -> None:
     """Saves an AST module to a file.
 
     Args:
         path: Destination file
         module: The AST module
+        format_with_black: ast.unparse is not PEP-8 compliant, so we apply black
+            on the result.
     """
     target = Path(path)
     target.parent.mkdir(parents=True, exist_ok=True)
     with target.open(mode="w", encoding="UTF-8") as file:
         file.write("# Automatically generated by Pynguin.\n")
-        file.write(
-            # ast.unparse is not PEP-8 compliant, so we apply black on the result.
-            black.format_str(
-                ast.unparse(ast.fix_missing_locations(module)), mode=black.FileMode()
-            )
-        )
+        output = ast.unparse(ast.fix_missing_locations(module))
+        if format_with_black:
+            # Import of black might cause problems if it is a SUT dependency,
+            # so we only import it if we need it.
+            import black  # pylint:disable=import-outside-toplevel
+
+            output = black.format_str(output, mode=black.FileMode())
+        file.write(output)
