@@ -26,7 +26,7 @@ import os
 import sys
 import threading
 from pathlib import Path
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, cast
 
 import pynguin.assertion.assertiongenerator as ag
 import pynguin.configuration as config
@@ -46,16 +46,22 @@ from pynguin.analyses.constants import (
     collect_static_constants,
 )
 from pynguin.analyses.module import generate_test_cluster
-from pynguin.analyses.types import TypeInferenceStrategy
+from pynguin.analyses.typesystem import TypeInferenceStrategy
 from pynguin.generation import export
-from pynguin.instrumentation.machinery import install_import_hook
+from pynguin.instrumentation.machinery import InstrumentationFinder, install_import_hook
+from pynguin.slicer.statementslicingobserver import StatementSlicingObserver
 from pynguin.testcase.execution import (
+    AssertionExecutionObserver,
     ExecutionTracer,
     ReturnTypeObserver,
     TestCaseExecutor,
 )
 from pynguin.utils import randomness
-from pynguin.utils.report import get_coverage_report, render_coverage_report
+from pynguin.utils.report import (
+    get_coverage_report,
+    render_coverage_report,
+    render_xml_coverage_report,
+)
 from pynguin.utils.statistics.runtimevariable import RuntimeVariable
 
 if TYPE_CHECKING:
@@ -143,6 +149,7 @@ def _setup_path() -> bool:
 
 
 def _setup_import_hook(
+    coverage_metrics: set[config.CoverageMetric],
     dynamic_constant_provider: DynamicConstantProvider | None,
 ) -> ExecutionTracer:
     _LOGGER.debug("Setting up instrumentation for %s", config.configuration.module_name)
@@ -151,6 +158,7 @@ def _setup_import_hook(
     install_import_hook(
         config.configuration.module_name,
         tracer,
+        coverage_metrics=coverage_metrics,
         dynamic_constant_provider=dynamic_constant_provider,
     )
     return tracer
@@ -245,7 +253,10 @@ def _setup_and_check() -> tuple[
     if not _setup_path():
         return None
     wrapped_constant_provider, dynamic_constant_provider = _setup_constant_seeding()
-    tracer = _setup_import_hook(dynamic_constant_provider)
+    tracer = _setup_import_hook(
+        set(config.configuration.statistics_output.coverage_metrics),
+        dynamic_constant_provider,
+    )
     if not _load_sut(tracer):
         return None
     if not _setup_report_dir():
@@ -257,7 +268,13 @@ def _setup_and_check() -> tuple[
         return None
     tracer.enable()
 
-    executor = TestCaseExecutor(tracer)
+    # Make alias to make the following lines shorter...
+    stop = config.configuration.stopping
+    executor = TestCaseExecutor(
+        tracer,
+        maximum_test_execution_timeout=stop.maximum_test_execution_timeout,
+        test_execution_time_per_statement=stop.test_execution_time_per_statement,
+    )
     _track_sut_data(tracer, test_cluster)
     _setup_random_number_generator()
     return executor, test_cluster, wrapped_constant_provider
@@ -316,12 +333,134 @@ def _get_coverage_ff_from_algorithm(
     return test_suite_coverage_func
 
 
+def _reload_instrumentation_loader(
+    coverage_metrics: set[config.CoverageMetric],
+    dynamic_constant_provider: DynamicConstantProvider | None,
+    tracer: ExecutionTracer,
+):
+    module_name = config.configuration.module_name
+    module = importlib.import_module(module_name)
+    tracer.current_thread_identifier = threading.current_thread().ident
+    first_finder = sys.meta_path[0]
+    assert isinstance(first_finder, InstrumentationFinder)
+    finder: InstrumentationFinder = first_finder
+    # pylint:disable=no-member
+    finder.update_instrumentation_metrics(
+        tracer=tracer,
+        coverage_metrics=coverage_metrics,
+        dynamic_constant_provider=dynamic_constant_provider,
+    )
+    importlib.reload(module)
+
+
+def _reset_cache_for_result(generation_result):
+    generation_result.invalidate_cache()
+    for test_case in generation_result.test_case_chromosomes:
+        test_case.invalidate_cache()
+        test_case.remove_last_execution_result()
+
+
+def _track_coverage_metrics(
+    executor: TestCaseExecutor,
+    generation_result: tsc.TestSuiteChromosome,
+    constant_provider: ConstantProvider,
+) -> set[config.CoverageMetric]:
+    """
+    Re-loads all required instrumentations for metrics that were not already
+    calculated and tracked during the result generation.
+    These metrics are then also calculated on the result, which is executed
+    once again with the new instrumentation.
+
+    Args:
+        executor: the testcase executor of the run
+        generation_result: the generated testsuite containing assertions
+        constant_provider: the constant provider required for the
+            reloading of the module
+
+    Returns:
+        The set of tracked coverage metrics, including the ones that we optimised for.
+    """
+    output_variables = config.configuration.statistics_output.output_variables
+    # Alias for shorter lines
+    cov_metrics = config.configuration.statistics_output.coverage_metrics
+    metrics_for_reinstrumenation: set[config.CoverageMetric] = set(cov_metrics)
+
+    to_calculate: list[tuple[RuntimeVariable, ff.TestSuiteCoverageFunction]] = []
+
+    if (
+        RuntimeVariable.LineCoverage in output_variables
+        and config.CoverageMetric.LINE not in cov_metrics
+    ):
+        metrics_for_reinstrumenation.add(config.CoverageMetric.LINE)
+        line_cov_ff = ff.TestSuiteLineCoverageFunction(executor)
+        to_calculate.append((RuntimeVariable.LineCoverage, line_cov_ff))
+
+    if (
+        RuntimeVariable.BranchCoverage in output_variables
+        and config.CoverageMetric.BRANCH not in cov_metrics
+    ):
+        metrics_for_reinstrumenation.add(config.CoverageMetric.BRANCH)
+        branch_cov_ff = ff.TestSuiteBranchCoverageFunction(executor)
+        to_calculate.append((RuntimeVariable.BranchCoverage, branch_cov_ff))
+
+    if RuntimeVariable.AssertionCheckedCoverage in output_variables:
+        metrics_for_reinstrumenation.add(config.CoverageMetric.CHECKED)
+        executor.set_instrument(True)
+        executor.add_observer(AssertionExecutionObserver(executor.tracer))
+        assertion_checked_coverage_ff = ff.TestSuiteAssertionCheckedCoverageFunction(
+            executor
+        )
+        to_calculate.append(
+            (RuntimeVariable.AssertionCheckedCoverage, assertion_checked_coverage_ff)
+        )
+
+    # re-instrument the files
+    dynamic_constant_provider = None
+    if isinstance(constant_provider, DynamicConstantProvider):
+        dynamic_constant_provider = constant_provider
+    _reload_instrumentation_loader(
+        metrics_for_reinstrumenation, dynamic_constant_provider, executor.tracer
+    )
+
+    # force new execution of the test cases after new instrumentation
+    _reset_cache_for_result(generation_result)
+
+    # set value for each newly calculated variable
+    for runtime_variable, coverage_ff in to_calculate:
+        generation_result.add_coverage_function(coverage_ff)
+        _LOGGER.info(f"Calculating resulting {runtime_variable.value}")
+        stat.track_output_variable(
+            runtime_variable, generation_result.get_coverage_for(coverage_ff)
+        )
+
+    # set new overall coverage
+    stat.track_output_variable(
+        RuntimeVariable.Coverage, generation_result.get_coverage()
+    )
+
+    ass_gen = config.configuration.test_case_output.assertion_generation
+    if (
+        ass_gen == config.AssertionGenerator.CHECKED_MINIMIZING
+        and RuntimeVariable.AssertionCheckedCoverage in output_variables
+    ):
+        _minimize_assertions(generation_result)
+
+    # reset whether to instrument tests and assertions as well as the SUT
+    instrument_test = config.CoverageMetric.CHECKED in cov_metrics
+    executor.set_instrument(instrument_test)
+    return metrics_for_reinstrumenation
+
+
 def _run() -> ReturnCode:
     if (setup_result := _setup_and_check()) is None:
         return ReturnCode.SETUP_FAILED
     executor, test_cluster, constant_provider = setup_result
     # Observe return types during execution.
     executor.add_observer(ReturnTypeObserver())
+    # traces slices for test cases after execution
+    coverage_metrics = config.configuration.statistics_output.coverage_metrics
+    if config.CoverageMetric.CHECKED in coverage_metrics:
+        executor.add_observer(StatementSlicingObserver(executor.tracer))
 
     algorithm: TestGenerationStrategy = _instantiate_test_generation_strategy(
         executor, test_cluster, constant_provider
@@ -340,19 +479,70 @@ def _run() -> ReturnCode:
     # search statistics
     executor.clear_observers()
 
-    _track_coverage_metrics(algorithm, generation_result)
+    _track_search_metrics(algorithm, generation_result)
+    _remove_statements_after_exceptions(generation_result)
+    _generate_assertions(executor, generation_result)
+    tracked_metrics = _track_coverage_metrics(
+        executor, generation_result, constant_provider
+    )
 
-    # Remove statements after exceptions
+    # Export the generated test suites
+    if (
+        config.configuration.test_case_output.export_strategy
+        == config.ExportStrategy.PY_TEST
+    ):
+        _export_chromosome(generation_result)
+
+    if config.configuration.statistics_output.create_coverage_report:
+        coverage_report = get_coverage_report(
+            generation_result,
+            executor,
+            tracked_metrics,
+        )
+        render_coverage_report(
+            coverage_report,
+            Path(config.configuration.statistics_output.report_dir) / "cov_report.html",
+            datetime.datetime.now(),
+        )
+        render_xml_coverage_report(
+            coverage_report,
+            Path(config.configuration.statistics_output.report_dir) / "cov_report.xml",
+            datetime.datetime.now(),
+        )
+    _collect_statistics(generation_result)
+    if not stat.write_statistics():
+        _LOGGER.error("Failed to write statistics data")
+    if generation_result.size() == 0:
+        # not able to generate one test case
+        return ReturnCode.NO_TESTS_GENERATED
+    return ReturnCode.OK
+
+
+def _remove_statements_after_exceptions(generation_result):
     truncation = pp.ExceptionTruncation()
     generation_result.accept(truncation)
     if config.configuration.test_case_output.post_process:
-
         unused_primitives_removal = pp.TestCasePostProcessor(
             [pp.UnusedStatementsTestCaseVisitor()]
         )
         generation_result.accept(unused_primitives_removal)
         # TODO(fk) add more postprocessing stuff.
 
+
+def _minimize_assertions(generation_result: tsc.TestSuiteChromosome):
+    _LOGGER.info("Minimizing assertions based on checked coverage")
+    assertion_minimizer = pp.AssertionMinimization()
+    generation_result.accept(assertion_minimizer)
+    stat.track_output_variable(
+        RuntimeVariable.Assertions, len(assertion_minimizer.remaining_assertions)
+    )
+    stat.track_output_variable(
+        RuntimeVariable.DeletedAssertions,
+        len(assertion_minimizer.deleted_assertions),
+    )
+
+
+def _generate_assertions(executor, generation_result):
     ass_gen = config.configuration.test_case_output.assertion_generation
     if ass_gen != config.AssertionGenerator.NONE:
         _LOGGER.info("Start generating assertions")
@@ -364,34 +554,8 @@ def _run() -> ReturnCode:
             generator = ag.AssertionGenerator(executor)
         generation_result.accept(generator)
 
-    # Export the generated test suites
-    if (
-        config.configuration.test_case_output.export_strategy
-        == config.ExportStrategy.PY_TEST
-    ):
-        _export_chromosome(generation_result)
 
-    if config.configuration.statistics_output.create_coverage_report:
-        render_coverage_report(
-            get_coverage_report(
-                generation_result,
-                executor,
-                config.configuration.statistics_output.coverage_metrics,
-            ),
-            Path(config.configuration.statistics_output.report_dir) / "cov_report.html",
-            datetime.datetime.now(),
-        )
-    _track_statistics(generation_result)
-    _collect_statistics()
-    if not stat.write_statistics():
-        _LOGGER.error("Failed to write statistics data")
-    if generation_result.size() == 0:
-        # not able to generate one test case
-        return ReturnCode.NO_TESTS_GENERATED
-    return ReturnCode.OK
-
-
-def _track_coverage_metrics(
+def _track_search_metrics(
     algorithm: TestGenerationStrategy, generation_result: tsc.TestSuiteChromosome
 ) -> None:
     """Track multiple set coverage metrics of the generated test suites.
@@ -406,22 +570,31 @@ def _track_coverage_metrics(
         RuntimeVariable.Coverage, generation_result.get_coverage()
     )
     coverage_metrics = config.configuration.statistics_output.coverage_metrics
-    if config.CoverageMetric.LINE in coverage_metrics:
-        line_coverage_ff: ff.CoverageFunction = _get_coverage_ff_from_algorithm(
-            algorithm, ff.TestSuiteLineCoverageFunction
-        )
-        stat.track_output_variable(
+    for metric, runtime, fitness_type in [
+        (
+            config.CoverageMetric.LINE,
             RuntimeVariable.LineCoverage,
-            generation_result.get_coverage_for(line_coverage_ff),
-        )
-    if config.CoverageMetric.BRANCH in coverage_metrics:
-        branch_coverage_ff: ff.CoverageFunction = _get_coverage_ff_from_algorithm(
-            algorithm, ff.TestSuiteBranchCoverageFunction
-        )
-        stat.track_output_variable(
+            ff.TestSuiteLineCoverageFunction,
+        ),
+        (
+            config.CoverageMetric.BRANCH,
             RuntimeVariable.BranchCoverage,
-            generation_result.get_coverage_for(branch_coverage_ff),
-        )
+            ff.TestSuiteBranchCoverageFunction,
+        ),
+        (
+            config.CoverageMetric.CHECKED,
+            RuntimeVariable.StatementCheckedCoverage,
+            ff.TestSuiteStatementCheckedCoverageFunction,
+        ),
+    ]:
+        if metric in coverage_metrics:
+            coverage_function: ff.CoverageFunction = _get_coverage_ff_from_algorithm(
+                algorithm, cast(type[ff.CoverageFunction], fitness_type)
+            )
+            stat.track_output_variable(
+                runtime,
+                generation_result.get_coverage_for(coverage_function),
+            )
 
 
 def _instantiate_test_generation_strategy(
@@ -435,7 +608,11 @@ def _instantiate_test_generation_strategy(
     return factory.get_search_algorithm()
 
 
-def _collect_statistics() -> None:
+def _collect_statistics(result: chrom.Chromosome) -> None:
+    stat.current_individual(result)
+    stat.track_output_variable(RuntimeVariable.Size, result.size())
+    stat.track_output_variable(RuntimeVariable.Length, result.length())
+
     stat.track_output_variable(
         RuntimeVariable.TargetModule, config.configuration.module_name
     )
@@ -451,12 +628,6 @@ def _collect_statistics() -> None:
         stat.set_output_variable_for_runtime_variable(runtime_variable, value)
 
 
-def _track_statistics(result: chrom.Chromosome) -> None:
-    stat.current_individual(result)
-    stat.track_output_variable(RuntimeVariable.Size, result.size())
-    stat.track_output_variable(RuntimeVariable.Length, result.length())
-
-
 def _export_chromosome(
     chromosome: chrom.Chromosome,
     file_name_suffix: str = "",
@@ -479,5 +650,9 @@ def _export_chromosome(
     )
     export_visitor = export.PyTestChromosomeToAstVisitor()
     chromosome.accept(export_visitor)
-    export.save_module_to_file(export_visitor.to_module(), target_file)
+    export.save_module_to_file(
+        export_visitor.to_module(),
+        target_file,
+        config.configuration.test_case_output.format_with_black,
+    )
     _LOGGER.info("Written %i test cases to %s", chromosome.size(), target_file)
