@@ -49,6 +49,12 @@ class RuntimeVariable(str, enum.Enum):
     # Obtained line coverage
     LineCoverage = "LineCoverage"
 
+    # Obtained checked coverage with no assertions
+    StatementCheckedCoverage = "StatementCheckedCoverage"
+
+    # Obtained checked coverage with assertions
+    AssertionCheckedCoverage = "AssertionCheckedCoverage"
+
     # The random seed used during the search.
     # A random one was used if none was specified in the beginning
     RandomSeed = "RandomSeed"
@@ -144,5 +150,12 @@ class RuntimeVariable(str, enum.Enum):
     # The mutation score
     MutationScore = "MutationScore"
 
+    # The number of assertions in the generated test suite
+    Assertions = "Assertions"
+
+    # The number of generated assertions that were removed, since
+    # they do not increase the resulting checked coverage
+    DeletedAssertions = "DeletedAssertions"
+
     def __repr__(self):
         return f"{self.name}"
